
% testing the different functions
clear; close all; clc

param.N = 200;               % number of discretization steps
param.x0 = 2;               % initial condition
param.T  = 5;               % terminal time
param.q  = 20;              % terminal weight
h = param.T/param.N;

Utst = rand(param.N,1);     % random control trajectory

% finite differences on nonlinear part
[F1, J1] = finite_difference(@Phi, Utst, param);

% imaginary trick 
[F2, J2] = i_trick(@Phi, Utst, param);

% forward AD
[F3, J3] = Phi_FAD(Utst, param);

% backward AD
[F4, J4] = Phi_BAD(Utst, param);

% running this script adds the casadi function Phi_casadi(U) to workspace
casadi_script
% evaluate the casadi function
[F5, J5] = Phi_casadi(Utst);


% check results
disp('Error between imaginary trick and finite differences:')
disp(max(max(abs(J2-J1))))

disp('Error between imaginary trick and forward AD:')
disp(max(max(abs(J2-J3))))

disp('Error between imaginary trick and backward AD:')
disp(max(max(abs(J2-J4))))

disp('Error between imaginary trick and casadi:')
disp(' ')
disp(max(max(abs(J2-J5))))